/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Testing button presses
 *  Copyright (c) 2011, Bob Mottram
 *  fuzzgun@gmail.com
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

#include <ros/ros.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream>
#include "serialport.cpp"
#include "buttonui.h"

using namespace std;

// Receive button presses from the Arduino
int get_button_press(
		     SerialPort &serial,
		     unsigned timeout_mS)
{
  const int bytes = 4;
  int index = -1;
  unsigned char comBuffer[bytes];
  int returned_bytes = serial.In(comBuffer, bytes, timeout_mS);
  if (returned_bytes > 0) {
    if (((char)comBuffer[0]>='0') && ((char)comBuffer[0]<='9')) {
      index = atoi((char*)comBuffer);
      int index2 = 0;
      for (index2=0;index2<sizeof(button_index)/sizeof(int);index2++) {
	if (button_index[index2]==index) break;
      }
      ROS_INFO("Button %d (%s) pressed", index,button_name[index2].c_str());
      index = index2;
    }
  }
  return index;
}

int main(int argc, char* argv[])
{
  ros::init(argc, argv, "buttonuitest");
  ros::NodeHandle n;
  ros::NodeHandle nh("~");

  std::string topic_path = "buttonui/";
  nh.getParam("topic_path", topic_path);

  // start and stop button digital inputs on the interface kit
  int i=-1;
  std::string str="";
  nh.getParam("startbuttonindex", i);
  if (i>-1) button_index[BUTTON_START] = i;
  i=-1;
  nh.getParam("stopbuttonindex", i);
  if (i>-1) button_index[BUTTON_STOP] = i;
  i=-1;
  nh.getParam("setposebuttonindex", i);
  if (i>-1) button_index[BUTTON_SET_POSE] = i;
  i=-1;
  nh.getParam("placebuttonindex", i);
  if (i>-1) button_index[BUTTON_PLACE] = i;
  i=-1;
  nh.getParam("learnbuttonindex", i);
  if (i>-1) button_index[BUTTON_LEARN] = i;
  i=-1;
  nh.getParam("gobuttonindex", i);
  if (i>-1) button_index[BUTTON_START] = i;

  // get the device to read from
  std::string device_name = "";
  nh.getParam("device", device_name);
  if (device_name == "") device_name = "/dev/ttyUSB0";

  // Communication with Arduino
  int baud_rate = 9600;
  std::string baud_rate_str = "";
  nh.getParam("baud", baud_rate_str);
  if (baud_rate_str!="") {
    baud_rate = atoi(baud_rate_str.c_str());
  }

  int data_bits = 8;
  std::string data_bits_str = "";
  nh.getParam("databits", data_bits_str);
  if (data_bits_str!="") {
    data_bits = atoi(data_bits_str.c_str());
  }

  int stop_bits = 1;
  std::string stop_bits_str = "";
  nh.getParam("stopbits", data_bits_str);
  if (stop_bits_str!="") {
    stop_bits = atoi(stop_bits_str.c_str());
  }

  SerialPort serial;
  if (serial.Open(device_name) == 0) {
    serial.Initialise(baud_rate, data_bits, stop_bits, PARITY_NONE);

    ROS_INFO("Connected to %s", device_name.c_str());

    ros::Rate loop_rate(10);
    while (n.ok()) {
      get_button_press(serial,100);
      ros::spinOnce();
      loop_rate.sleep();
    }

    ROS_INFO("Closing %s", device_name.c_str());
  }
  else {
    ROS_WARN("Can't open %s", device_name.c_str());
  }

  return 0;
}

